<?php
/**
 * Extend core customize section with extra controls.
 *
 * @package    Community_Builder
 * @subpackage Customizer
 * @copyright  Copyright (c) 2018, Brajesh Singh
 * @license    https://www.gnu.org/licenses/gpl.html GNU Public License
 * @author     Brajesh Singh
 */

// Do not allow direct access over web.
defined( 'ABSPATH' ) || exit;

/**
 * Prebuild customizer sections extension with extra controls.
 */
class CB_Customize_Core_Features_Extension {

	/**
	 * Boot itself
	 *
	 * @param WP_Customize_Manager $wp_customize manager.
	 *
	 * @return CB_Customize_Core_Features_Extension
	 */
	public static function boot( $wp_customize ) {

		$self = new self();
		$self->setup( $wp_customize );

		return $self;
	}

	/**
	 * Setup.
	 *
	 * @param WP_Customize_Manager $wp_customize manager.
	 */
	private function setup( $wp_customize ) {
		// move core WordPress Sections under various panels.
		$this->add_mobile_logo( $wp_customize );
		$this->add_site_tagline_toggle( $wp_customize );
		$this->add_site_background_color( $wp_customize );
		$this->add_page_header_controls( $wp_customize );
	}

	/**
	 * Add mobile logo.
	 *
	 * @param WP_Customize_Manager $wp_customize manager.
	 */
	private function add_mobile_logo( $wp_customize ) {

		$wp_customize->add_setting(
			'mobile_logo',
			array(
				'theme_supports' => array( 'cb-mobile-logo' ),
				'transport'      => 'postMessage',
				'sanitize_callback' => 'absint',
			)
		);

		$custom_logo_args = get_theme_support( 'cb-mobile-logo' );
		$wp_customize->add_control(
			new WP_Customize_Cropped_Image_Control(
				$wp_customize,
				'mobile_logo',
				array(
					'label'         => __( 'Mobile Logo', 'community-builder' ),
					'section'       => 'title_tagline',
					'priority'      => 9,
					'height'        => $custom_logo_args[0]['height'],
					'width'         => $custom_logo_args[0]['width'],
					'flex_height'   => $custom_logo_args[0]['flex-height'],
					'flex_width'    => $custom_logo_args[0]['flex-width'],
					'button_labels' => array(
						'select'       => __( 'Select logo', 'community-builder' ),
						'change'       => __( 'Change logo', 'community-builder' ),
						'remove'       => __( 'Remove', 'community-builder' ),
						'default'      => __( 'Default', 'community-builder' ),
						'placeholder'  => __( 'No logo selected', 'community-builder' ),
						'frame_title'  => __( 'Select logo', 'community-builder' ),
						'frame_button' => __( 'Choose logo', 'community-builder' ),
					),
				)
			)
		);
		// Register sections.
		$wp_customize->add_section(
			new CB_Customize_Section_Pro(
				$wp_customize,
				'cb-pro-link',
				array(
					'title'    => '', //esc_html__( 'Community Builder Pro', 'community-builder' ),
					'pro_text' => esc_html__( 'More Options with Community Builder Pro', 'community-builder' ),
					'pro_url'  => 'https://buddydev.com/themes/community-builder-pro/?utm_source=customizer&utm_campaign=CommunityBuilderPro',
					'priority' => 1,
				)
			)
		);
	}

	/**
	 * Add Tagline toggle option.
	 *
	 * @param WP_Customize_Manager $wp_customize customize manager.
	 */
	private function add_site_tagline_toggle( $wp_customize ) {

		$wp_customize->add_setting(
			'show-tagline',
			array(
				'type'              => 'theme_mod',
				'capability'        => 'edit_theme_options',
				//	'transport'         => 'postMessage', // Previewed with JS in the Customizer controls window.
				'default'           => cb_get_default( 'show-tagline' ),
				'sanitize_callback' => array( 'CB_Data_Sanitizer', 'sanitize_int' ),
			)
		);

		$wp_customize->add_control(
			'show-tagline',
			array(
				'label'       => __( 'Show Tagline', 'community-builder' ),
				'description' => __( 'Show tagline in header.', 'community-builder' ),
				'section'     => 'title_tagline',
				'type'        => 'checkbox',
				'priority'    => 42,
			)
		);
	}

	/**
	 * Add site background colors.
	 *
	 * @param WP_Customize_Manager $wp_customize customize manager.
	 */
	private function add_site_background_color( $wp_customize ) {

		$wp_customize->add_setting(
			'background-color',
			array(
				'type'              => 'theme_mod',
				'capability'        => 'edit_theme_options',
				'transport'         => 'postMessage', // Previewed with JS in the Customizer controls window.
				'default'           => cb_get_default( 'background-color' ),
				'sanitize_callback' => array( 'CB_Data_Sanitizer', 'sanitize_alpha_color' ),
			)
		);

		$wp_customize->add_control(
			new CB_Customize_Control_Color(
				$wp_customize,
				'background-color',
				array(
					'label'       => __( 'Background Color', 'community-builder' ),
					'description' => '',
					'section'     => 'background_image',
					'priority'    => 70,
				)
			)
		);
	}

	/**
	 * Add Headers
	 *
	 * @param WP_Customize_Manager $wp_customize manager.
	 */
	private function add_page_header_controls( $wp_customize ) {

		$this->add_page_header_dimension(
			$wp_customize,
			array(
				'option_name' => 'page-header-height',
				'label'       => __( 'Page Header Height', 'community-builder' ),
				'description' => __( 'Page header height', 'community-builder' ),
				'priority'    => 1,
				'default'     => cb_get_default( 'page-header-height' ),
			)
		);

		$this->add_page_header_background_color(
			$wp_customize,
			array(
				'option_name' => 'page-header-mask-color',
				'label'       => __( 'Mask Color', 'community-builder' ),
				'description' => __( 'Used to mask the page header image', 'community-builder' ),
				'priority'    => 2,
				'default'     => cb_get_default( 'page-header-mask-color' ),
			)
		);

		$this->add_page_header_background_color(
			$wp_customize,
			array(
				'option_name' => 'page-header-background-color',
				'label'       => __( 'Background Color', 'community-builder' ),
				'description' => '',
				'priority'    => 2,
				'default'     => cb_get_default( 'page-header-background-color' ),
			)
		);
		$this->add_page_header_border(
			$wp_customize,
			array(
				'option_name' => 'page-header-border',
				'label'       => __( 'Border', 'community-builder' ),
				'description' => '',
				'priority'    => 3,
				'default'     => cb_get_default( 'page-header-border' ),
			)
		);

		$this->add_page_header_color(
			$wp_customize,
			array(
				'option_name' => 'page-header-title-text-color',
				'label'       => __( 'Title Color', 'community-builder' ),
				'description' => '',
				'priority'    => 4,
				'default'     => cb_get_default( 'page-header-title-text-color' ),
			)
		);

		$this->add_page_header_color(
			$wp_customize,
			array(
				'option_name' => 'page-header-content-text-color',
				'label'       => __( 'Description Color', 'community-builder' ),
				'description' => '',
				'priority'    => 5,
				'default'     => cb_get_default( 'page-header-content-text-color' ),
			)
		);

		$this->add_page_header_color(
			$wp_customize,
			array(
				'option_name' => 'page-header-meta-text-color',
				'label'       => __( 'Meta text color', 'community-builder' ),
				'description' => __( 'Page header meta text color', 'community-builder' ),
				'priority'    => 6,
				'default'     => cb_get_default( 'page-header-meta-text-color' ),
			)
		);

		$this->add_page_header_color(
			$wp_customize,
			array(
				'option_name' => 'page-header-meta-link-color',
				'label'       => __( 'Meta link color', 'community-builder' ),
				'description' => __( 'Page header meta text color', 'community-builder' ),
				'priority'    => 7,
				'default'     => cb_get_default( 'page-header-meta-link-color' ),
			)
		);

		$this->add_page_header_color(
			$wp_customize,
			array(
				'option_name' => 'page-header-meta-link-hover-color',
				'label'       => __( 'Meta link hover color', 'community-builder' ),
				'description' => __( 'Page header meta text color', 'community-builder' ),
				'priority'    => 8,
				'default'     => cb_get_default( 'page-header-meta-link-hover-color' ),
			)
		);
	}


	/**
	 * Add background color.
	 *
	 * @param WP_Customize_Manager $wp_customize customize manager.
	 * @param array                $args args.
	 */
	private function add_page_header_dimension( $wp_customize, $args = array() ) {

		$config = CB_Customize_Setting_Builder::get_responsive_range_settings(
			array(
				'input_attrs' => array(
					'min'  => 0,
					'max'  => 1000,
					'step' => 1,
				),
				'default'     => $args['default'],
				'label'       => $args['label'],
			)
		);

		$settings = array_merge(
			array(
				'type'       => 'theme_mod',
				'capability' => 'edit_theme_options',
			),
			$config['setting']
		);

		$control_options = $config['control'];

		$wp_customize->add_setting(
			$args['option_name'],
			array(
				'type'              => 'theme_mod',
				'capability'        => 'edit_theme_options',
				'sanitize_callback' => $settings['sanitize_callback'],
				'transport'         => $settings['transport'],
				'default'           => $settings['default'],
			)
		);

		$wp_customize->add_control(
			new CB_Customize_Control_Range_Responsive(
				$wp_customize,
				$args['option_name'],
				array(
					'label'           => $args['label'],
					'description'     => $args['description'],
					'section'         => 'header_image',
					'priority'        => $args['priority'],
					'input_attrs'     => $control_options['input_attrs'],
					'active_callback' => $control_options['active_callback'],

				)
			)
		);
	}

	/**
	 * Add background color.
	 *
	 * @param WP_Customize_Manager $wp_customize customize manager.
	 * @param array                $args args.
	 */
	private function add_page_header_background_color( $wp_customize, $args = array() ) {

		$wp_customize->add_setting(
			$args['option_name'],
			array(
				'type'              => 'theme_mod',
				'capability'        => 'edit_theme_options',
				'default'           => $args['default'],
				'transport'         => isset( $args['transport'] ) ? $args['transport'] : 'postMessage',
				// Previewed with JS in the Customizer controls window.
				'sanitize_callback' => array( 'CB_Data_Sanitizer', 'sanitize_alpha_color' ),
			)
		);

		$wp_customize->add_control(
			new CB_Customize_Control_Color(
				$wp_customize,
				$args['option_name'],
				array(
					'label'       => $args['label'],
					'description' => $args['description'],
					'section'     => 'header_image',
					'priority'    => $args['priority'],

				)
			)
		);
	}

	/**
	 * Add color section.
	 *
	 * @param WP_Customize_Manager $wp_customize customize manager.
	 * @param array                $args args.
	 */
	private function add_page_header_color( $wp_customize, $args = array() ) {

		$wp_customize->add_setting(
			$args['option_name'],
			array(
				'type'              => 'theme_mod',
				'capability'        => 'edit_theme_options',
				'default'           => $args['default'],
				'transport'         => isset( $args['transport'] ) ? $args['transport'] : 'postMessage',
				// Previewed with JS in the Customizer controls window.
				'sanitize_callback' => array( 'CB_Data_Sanitizer', 'sanitize_alpha_color' ),
			)
		);

		$wp_customize->add_control(
			new CB_Customize_Control_Color(
				$wp_customize,
				$args['option_name'],
				array(
					'label'       => $args['label'],
					'description' => $args['description'],
					'section'     => 'header_image',
					'priority'    => $args['priority'],

				)
			)
		);
	}

	/**
	 * Add color section.
	 *
	 * @param WP_Customize_Manager $wp_customize customize manager.
	 * @param array                $args args.
	 */
	private function add_page_header_border( $wp_customize, $args = array() ) {

		$wp_customize->add_setting(
			$args['option_name'],
			array(
				'type'              => 'theme_mod',
				'capability'        => 'edit_theme_options',
				'default'           => $args['default'],
				'transport'         => isset( $args['transport'] ) ? $args['transport'] : 'postMessage',
				// Previewed with JS in the Customizer controls window.
				'sanitize_callback' => array( 'CB_Data_Sanitizer', 'sanitize_border' ),
			)
		);

		$wp_customize->add_control(
			new CB_Customize_Control_Border(
				$wp_customize,
				$args['option_name'],
				array(
					'label'          => $args['label'],
					'description'    => $args['description'],
					'linked_choices' => true,
					'section'        => 'header_image',
					'priority'       => $args['priority'],
				)
			)
		);
	}
}
